<?php

namespace QUI\RabbitMQServer;

use PCSG\Tasks\TestTask\TestTaskData;
use Psr\Log\LogLevel;
use QUI\RabbitMQServer\Interface\TaskSchedulerInterface;
use QUI\System\License;
use QUI\Utils\Uuid;

use function date;
use function file_exists;
use function sleep;

use const VAR_DIR;

class HeartbeatService
{
    public function __construct(private ?TaskSchedulerInterface $taskScheduler = null)
    {
        if (is_null($this->taskScheduler)) {
            $provider = new Provider();
            $this->taskScheduler = $provider->getTaskScheduler();
        }
    }

    /**
     * Checks for a special heartbeat identifiert in today's rabbit heartbeat log.
     *
     * @param string|null $heartbeatTaskIdentifier (optional)
     * @param int $timeoutSeconds (optional) [default: 10 seconds]
     * @return bool
     */
    public function checkHeartbeat(?string $heartbeatTaskIdentifier = null, int $timeoutSeconds = 10): bool
    {
        if (is_null($heartbeatTaskIdentifier)) {
            $heartbeatTaskIdentifier = Uuid::get();
        }

        $testTaskData = new TestTaskData(License::getSystemId(), $heartbeatTaskIdentifier);
        $this->taskScheduler->queueWorkerTask($testTaskData, LogLevel::DEBUG);

        $secondsWaited = 0;

        while (true) {
            // File is specified in while loop for edge cases, where the day switches at 00:00 a.m.
            $logFile = VAR_DIR . 'log/rabbitmqserver_heartbeat-' . date('Y-m-d') . '.log';

            if (file_exists($logFile)) {
                $grep = `grep "{$heartbeatTaskIdentifier}" {$logFile}`;
            }

            if (!empty($grep)) {
                return true;
            }

            sleep(1);

            if (++$secondsWaited >= $timeoutSeconds) {
                return false;
            }
        }
    }
}
