<?php

namespace QUI\RabbitMQServer\Logger;

use PCSG\RabbitMQ\ApiClient\Entity\LogEntry;
use Psr\Log\LogLevel;
use QUI;

use function array_merge;
use function date;
use function file_put_contents;

/**
 * Logs task specific events.
 */
readonly class Logger
{
    public function __construct(
        private LogType $logType
    ) {
    }

    public function log(LogEntry $logEntry): void
    {
        QUI\System\Log::write(
            $logEntry->message,
            $this->getQuiqqerLogLevel($logEntry),
            array_merge(
                $logEntry->context ?: [],
                [
                    'originalLogDate' => $logEntry->date
                ]
            ),
            $this->getLogFileName($logEntry)
        );
    }

    /**
     * Logs to a special heartbeat logfile that is independent of QUIQQER log levels.
     *
     * @param string $testTaskIdentifier - The identifiert of the test task
     * @return void
     */
    public function logHeartbeat(string $testTaskIdentifier): void
    {
        $logDir = VAR_DIR . 'log/';
        $logFile = $logDir . 'rabbitmqserver_heartbeat-' . date('Y-m-d') . '.log';

        QUI\Utils\System\File::mkfile($logFile);

        file_put_contents($logFile, '[' . date('Y-m-d H:i:s') . '] ' . $testTaskIdentifier . "\n", FILE_APPEND);
    }

    /**
     * Get log file name for QUIQQER logger.
     *
     * @param LogEntry $logEntry
     * @return string
     */
    private function getLogFileName(LogEntry $logEntry): string
    {
        return $this->logType->value . '.' . mb_strtolower($logEntry->logLevel);
    }

    /**
     * Map PSR-3 log level to QUIQQER log level.
     *
     * @param LogEntry $logEntry
     * @return int
     */
    private function getQuiqqerLogLevel(LogEntry $logEntry): int
    {
        return match ($logEntry->logLevel) {
            LogLevel::INFO => QUI\System\Log::LEVEL_INFO,
            LogLevel::NOTICE => QUI\System\Log::LEVEL_NOTICE,
            LogLevel::WARNING => QUI\System\Log::LEVEL_WARNING,
            LogLevel::ERROR => QUI\System\Log::LEVEL_ERROR,
            LogLevel::CRITICAL => QUI\System\Log::LEVEL_CRITICAL,
            LogLevel::ALERT => QUI\System\Log::LEVEL_ALERT,
            LogLevel::EMERGENCY => QUI\System\Log::LEVEL_EMERGENCY,
            default => QUI\System\Log::LEVEL_DEBUG
        };
    }
}
