<?php

namespace QUI\RabbitMQServer\REST\Route;

use PCSG\RabbitMQ\ApiClient\Entity\LogEntry;
use PCSG\RabbitMQ\ApiClient\Entity\TaskLog;
use PCSG\Tasks\TestTask\TestTaskData;
use Psr\Http\Message\ResponseInterface as ResponseInterface;
use QUI\RabbitMQServer\Logger\Logger;
use QUI\RabbitMQServer\Logger\LogType;

use function mb_strpos;
use function str_replace;

class WriteTaskLog extends AbstractRoute
{
    private readonly Logger $logger;

    public function __construct()
    {
        $this->logger = new Logger(LogType::TASK_LOG);
    }

    protected function handleRequest(): ResponseInterface
    {
        try {
            /** @var TaskLog $taskLog */
            $taskLog = $this->getInstanceFromRequest(TaskLog::class);

            foreach ($taskLog->logEntries as $logEntry) {
                if (mb_strpos($logEntry->message, TestTaskData::HEARTBEAT_LOG_PREFIX) === 0) {
                    $this->logger->logHeartbeat(
                        str_replace(TestTaskData::HEARTBEAT_LOG_PREFIX, '', $logEntry->message)
                    );

                    continue;
                }

                $message = "[$taskLog->workerClass :: $taskLog->taskUUID] $logEntry->message";

                $loggerLogEntry = new LogEntry(
                    $message,
                    $logEntry->context,
                    $logEntry->logLevel,
                    $logEntry->date
                );

                $this->logger->log($loggerLogEntry);
            }
        } catch (\Exception $exception) {
            return $this->buildExceptionErrorResponse($exception);
        }

        return $this->buildSuccessResponse('Log entry written.');
    }
}
